#!/bin/sh

doOptional=false
useDebianSource=false

fromearly=false
fromdevelop=false
domerge=false

. ./build-env.sh

lxmmmkb=`echo ${lxdebkb} | cut -d '.' -f 1-2`

if [ ! -e linux-${lxmmmkb}.y ]; then
  #git clone -b linux-${lxmmmkb}.y https://git.kernel.org/pub/scm/linux/kernel/git/stable/linux-stable.git linux-${lxmmmkb}.y
  #git clone --depth 100 https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git linux-${lxmmmkb}.y
  git clone -b mainline-${lxmmmkb} https://github.com/scpcom/linux linux-${lxmmmkb}.y
fi

echo "Copying files from linux-${lxmmmkb}.y git..."
mkdir linux && \
cp -apP linux-${lxmmmkb}.y/. linux/ && \
cd linux

VERSION=`cat Makefile | grep -E '^VERSION.=' | cut -d "=" -f 2- | tr -d " \t\n\r"`
PATCHLEVEL=`cat Makefile | grep PATCHLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`
SUBVERSION=`cat Makefile | grep SUBLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`

quiltpatches() {
  if [ -e ../patches/$2 ]; then
    mkdir -p patches/$1

    for p in `cat ../patches/$2 | grep -v '^#' | grep patch` ; do
      b=`basename $p`

      if [ -e ../patches/${p} ]; then
        cp -p ../patches/${p} patches/${p}
      else
        cp -p  ../patches/${b} patches/${p}
      fi
    done

    cat ../patches/$2 >> patches/series

    quilt push -a
  fi

}

simplepatch() {
  p=$1
  f=$2

  b=`basename $p`
  d=`dirname $p`
  l=${p}
  n=`echo $b | sed s/\.patch$//g`

  mkdir -p patches/${d}

  if [ -e ../patches/${p} ]; then
    cp -p ../patches/${p} patches/${p}
  else
    cp -p  ../patches/${b} patches/${p}
  fi

  if echo $n | grep -q -E '^mali-' ; then
    if [ -e drivers/gpu/arm/mali400 ]; then
      sed -i "s|/src/devicedrv/|/drivers/gpu/arm/mali400/|g" patches/${p}
    else
      sed -i "s|/src/devicedrv/|/drivers/gpu/arm/|g" patches/${p}
    fi
  fi

  if echo $n | grep -q -E '^linux-' ; then
    n=`echo $n | sed s/^linux-4\.1.\.y-//g`
    n=`echo $n | sed s/^linux-4\.4\.y-//g`
    n=`echo $n | sed s/^linux-4\.1.-//g`

  elif echo $n | grep -q -E '^mali-' ; then
    n=`echo $n | sed s/^mali-r.p.-//g`
  fi

  git apply patches/${p}
  git add ${f}
  git commit -m "${d} ${n}"
  echo "${p}" >> patches/series
}

gitpatches() {
  if [ -e ../patches/$2 ]; then
    echo ""
    echo "Apply series patches/$2"
    echo ""

    mkdir -p patches/$1

    l=none
    for p in `cat ../patches/$2 | grep -v '^#' | grep patch` ; do
      b=`basename $p`
      d=`dirname $p`
      l=${p}
      n=`echo $b | sed s/\.patch$//g`

      if [ -e ../patches/${p} ]; then
        cp -p ../patches/${p} patches/${p}
      else
        cp -p  ../patches/${b} patches/${p}
      fi

      echo "Apply patch patches/${p}"

      #if grep -q '\\r' patches/${p} ; then
      #  sed -i -e 's/\r$//g' patches/${p}
      #fi

      if echo $n | grep -q -E '^mali-' ; then
        if [ -e drivers/gpu/arm/mali400 ]; then
          sed -i "s|/src/devicedrv/|/drivers/gpu/arm/mali400/|g" patches/${p}
        else
          sed -i "s|/src/devicedrv/|/drivers/gpu/arm/|g" patches/${p}
        fi
      fi

      nogitcp=1
      gitcpcont=0
      nogitam=1
      gitamcont=0
      if false ; then
      if grep -m1 -q "^From ${n} Mon Sep 17 00:00:00 2001" patches/${p} ; then
        rc=0
        git cherry-pick ${n} || rc=$?
        echo "git cherry-pick ${n}: rc=${rc}"
        if [ $rc = 0 ] ; then
          nogitcp=0
          echo ${p} >> patches/series-cherry-pick
        elif [ $rc != 128 ] ; then
          #gitcpcont=1
          git cherry-pick --abort
        fi
      fi
      fi

      if [ $nogitcp = 1 -a $gitcpcont = 0 ]; then
      if grep -m1 -q "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} ; then
        c=`grep -m1 "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} | sed s/'^From '//g | cut -d ' ' -f1`

        sed -i -e "0,/^---$/ s/^---$/\n${d} ${c}\n---/" patches/${p}

        if git am --keep-cr < patches/${p} ; then
          nogitam=0
          echo ${p} >> patches/series-am
        else
          gitamcont=1
          #git am --skip
        fi
      fi
      fi

      if [ $nogitam = 1 -a $nogitcp = 1 ]; then
        pb=`echo ${p} | sed s/'\.patch$'/'-bin.patch'/g`

        #echo "Check patches/${pb}"
        if [ -e ../patches/${pb} ]; then
          cp -p ../patches/${pb} patches/${pb}
          echo git apply --stat --apply patches/${pb}
          git apply --stat --apply patches/${pb}
        fi

        if ! git apply --stat --apply patches/${p} ; then
          #git apply --stat --apply --3way patches/${p}
          #sed -i -e 's/\r$//g' patches/${p}
          patch -p1 --no-backup-if-mismatch < patches/${p}
          echo ${p} >> patches/series-no-git
        else
          echo ${p} >> patches/series-no-am
        fi
      fi

      if echo $n | grep -q -E '^linux-' ; then
        n=`echo $n | sed s/^linux-4\.1.\.y-//g`
        n=`echo $n | sed s/^linux-4\.4\.y-//g`
        n=`echo $n | sed s/^linux-4\.1.-//g`

      elif echo $n | grep -q -E '^mali-' ; then
        n=`echo $n | sed s/^mali-r.p.-//g`

      elif grep -m1 -q "^From ${n} Mon Sep 17 00:00:00 2001" patches/${p} ; then
        if ! grep -q '^---$' patches/${p} ; then
          exit 1
        fi

      else
        exit 1
      fi

      if [ $nogitam = 1 -a $nogitcp = 1 ]; then
        echo git commit -m "${d} ${n}"
        git add .
        #git commit -m "${d} ${n}"
        #echo "${d} ${n}" > patches/${p}.msg

        if grep -m1 -q "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} ; then
          if ! grep -q '^---$' patches/${p} ; then
            exit 1
          fi
          cat patches/${p} | grep -m1 -B399 '^---$' | sed /^---$/d | grep -m1 -A399 '^Subject:' | sed s/'^Subject: '/''/g > patches/${p}.msg
        else
          echo "${d} ${n}" > patches/${p}.msg
        fi
        
        if [ $gitcpcont = 1 ]; then
          git cherry-pick --continue
        elif [ $gitamcont = 1 ]; then
          git am --continue
        else
          git commit -F patches/${p}.msg
        fi
      fi

      echo ""
    done

    echo "Now in patch patches/${l}"
    cat ../patches/$2 >> patches/series

    #quilt push -a
  fi

}

applypatches() {
  gitpatches $1 $2
  #quiltpatches $1 $2
}

gitmerge() {
  mname=$1
  mbranch=$2

  start_range=`git rev-list --simplify-by-decoration -2 HEAD`

  start_range_0=`echo $start_range | cut -d ' ' -f 1`
  start_range_1=`echo $start_range | cut -d ' ' -f 2`

  #git merge -s recursive -X ours -X renormalize -X no-renames origin/${mbranch} | tee ../merge-${mname}.log
  git merge -s recursive -X ours -X renormalize -X no-renames ${mbranch} | tee ../merge-${mname}.log

  commit_range=`git rev-list --simplify-by-decoration -2 HEAD`

  commit_range_0=`echo $commit_range | cut -d ' ' -f 1`
  commit_range_1=$start_range_0

  git log --decorate=no --reverse ${commit_range_1}..${commit_range_0} | grep -E '^commit ' | cut -d ' ' -f 2- > ../merge-${mname}-commit-list.txt

  cm=`cat ../merge-${mname}-commit-list.txt | tail -1`

  for c in `cat ../merge-${mname}-commit-list.txt` ; do
    if [ ${c} != ${cm} ] ; then
      echo "${mname}/${c}.patch" >> patches/series-${mname}-merge
    fi
    if [ ${mname} = mainline ]; then
      grep ${c} ../patches/series-${mname}-??? | cut -d ':' -f 2- >> patches/series-${mname}-quilt
    else
      grep ${c} ../patches/series-${mname}-20?????? | cut -d ':' -f 2- >> patches/series-${mname}-quilt
    fi
  done

  cat patches/series-${mname}-merge >> patches/series-merge
  cat patches/series-${mname}-merge >> patches/series
}

lxmergever=${lxmmmkb}
#git checkout -b mainline-${lxmergever}

if $fromdevelop ; then
  git checkout mainline-${lxmmmkb}
  git checkout odroid-${lxmmmkb}.y

  git checkout -b mesonsbc-${lxmmmkb}.y
else
  git checkout mesonsbc-${lxmmmkb}.y
fi

VERSION=`cat Makefile | grep -E '^VERSION.=' | cut -d "=" -f 2- | tr -d " \t\n\r"`
PATCHLEVEL=`cat Makefile | grep PATCHLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`
SUBVERSION=`cat Makefile | grep SUBLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`

if $fromearly ; then
mkdir -p patches
mkdir -p patches/platform

for f in `cat ../patches/series-odroidc2 | grep -E '^platform/' | cut -d / -f 2-` ; do
  cp -p ../patches/${f} patches/platform/
done

cat ../patches/series-odroidc2 >> patches/series

if [ $doOptional = true ]; then
  for f in `cat ../patches/series-odroidc2-optional | grep -E '^platform/' | cut -d / -f 2-` ; do
    cp -p ../patches/${f} patches/platform/
  done

  cat ../patches/series-odroidc2-optional >> patches/series
fi

cp -p ../patches/stmmac_platform_mac_addr_set.patch patches/
echo "stmmac_platform_mac_addr_set.patch" >> patches/series

quilt push -a

if [ $doOptional = true ]; then
  for f in `cat ../patches/series-odroidc2-binary` ; do
    git apply ../patches/${f}
  done
fi
fi

if $fromdevelop ; then
applypatches platform series-odroidc2

simplepatch \
 platform/linux-5.4.y-stmmac_platform_mac_addr_set.patch \
 drivers/net/ethernet/stmicro/stmmac/stmmac_platform.c
fi

#git merge -s recursive -X ours -X renormalize -X no-renames mainline-${lxmergever} | tee ../merge.log

if [ -e ../patches/mainline ]; then
  #mv patches/platform/mainline patches/
  for s in `ls ../patches/series-mainline-???` ; do
    b=`basename $s`
    v=`basename $s | cut -d '-' -f 3`
    if [ $SUBVERSION -lt $v ]; then
      echo ""
      if [ -e ../patches/${b}-prepare ]; then
        applypatches mainline ${b}-prepare
      fi
      #applypatches mainline ${b}
      #echo ""
      #echo "" >> patches/series
      #cat $s >> patches/series
      #quilt push -a
    fi
  done
fi

if $domerge ; then
gitmerge mainline mainline-${lxmergever}
fi

if [ -e ../patches/mainline ]; then
  #mv patches/platform/mainline patches/
  for s in `ls ../patches/series-mainline-???` ; do
    b=`basename $s`
    v=`basename $s | cut -d '-' -f 3`
    #echo "$SUBVERSION -lt $v"
    if [ $SUBVERSION -lt $v ]; then
      echo ""
      if [ -e ../patches/${b}-post ]; then
        applypatches mainline ${b}-post
      fi
      #echo ""
      #echo "" >> patches/series
      #cat $s >> patches/series
      #quilt push -a
    fi
  done
fi

if $fromdevelop ; then
git revert --no-edit 8360063bfae53a765464aa39755b49cf736f1df0

applypatches platform series-cpuinfo

if [ -e drivers/gpu/arm ]; then
  simplepatch \
    r7p0/mali-r6p2-2008-umplock_fix_build.patch \
    drivers/gpu/arm
fi

if [ -e drivers/gpu/arm ]; then
  simplepatch \
    r7p0/mali-r6p2-2009-fix_5.4_build.patch \
    drivers/gpu/arm
fi
fi

if [ -e drivers/gpu/arm ]; then
  simplepatch \
    r7p0/mali-r6p2-2012-mali_dt_broken.patch \
    drivers/gpu/arm
fi

simplepatch \
  platform/linux-5.4.y-odroid-n1-add_rk3399_restart_driver.patch \
  drivers/power/reset

git am --keep-cr < ../patches/linux-5.4.y-0001-arm64-dts-rockchip-ODROID-N1-import.patch
git am --keep-cr < ../patches/linux-5.4.y-0002-arm64-dts-rockchip-ODROID-N1-remove-unused-nodes.patch
git am --keep-cr < ../patches/linux-5.4.y-0003-arm64-dts-rockchip-ODROID-N1-use-pin-names.patch
git am --keep-cr < ../patches/linux-5.4.y-0004-arm64-dts-rockchip-ODROID-N1-rename-pins-and-regulat.patch
git am --keep-cr < ../patches/linux-5.4.y-0005-arm64-dts-rockchip-ODROID-N1-update.patch

pcount=0
if [ -e patches/series ]; then
  pcount=`cat patches/series | grep -v -E '^#' | grep patch | wc -l`

  mv patches/series patches/series-git
fi

gcount=0
if [ -e patches/series-merge ]; then
  gcount=`cat patches/series-merge | grep -v -E '^#' | grep patch | wc -l`
fi
mcount=0
if [ -e patches/series-am ]; then
  mcount=`cat patches/series-am | grep -v -E '^#' | grep patch | wc -l`
fi
acount=0
if [ -e patches/series-no-am ]; then
  acount=`cat patches/series-no-am  | grep -v -E '^#' | grep patch | wc -l`
fi
ncount=0
if [ -e patches/series-no-git ]; then
  ncount=`cat patches/series-no-git | grep -v -E '^#' | grep patch | wc -l`
fi

echo ""

if [ $gcount -gt 0 -o $mcount -gt 0 -o $acount -gt 0 -o $ncount -gt 0 ]; then
  echo "Applied ${pcount} patches (${gcount} with git merge, ${mcount} with git am, ${acount} with git apply, ${ncount} outside git)."

else
  echo "Applied ${pcount} patches."
fi

echo ""

if false ; then
cp -p ../patches/*_defconfig arch/arm64/configs/

#LANG=C git diff arch/arm64/configs
git add arch/arm64/configs
git commit -m "arm64: configs: add meson defconfigs"
fi

cfgmode=none
for f in ../patches/*_defconfig ; do
  c=`basename $f`
  c=${c/_defconfig/}

  if grep -q -E '^CONFIG_ARM64=y|^CONFIG_ARM64_.*=y' $f ; then
    #if [ -e arch/arm64/configs/${c}_defconfig ]; then
    # mv arch/arm64/configs/${c}_defconfig arch/arm64/configs/${c}_git_defconfig
    #fi
    if [ -e arch/arm64/configs/${c}_defconfig ]; then
      if ! diff -q $f arch/arm64/configs/${c}_defconfig ; then
        cfgmode=update
      fi
    else
      if [ "${cfgmode}" = "none" ]; then
        cfgmode=add
      fi
    fi
    cp -p $f arch/arm64/configs/
  fi
done

if [ "${cfgmode}" != "none" ]; then
  #LANG=C git diff arch/arm64/configs
  git add arch/arm64/configs
  git commit -m "arm64: configs: $cfgmode meson defconfigs"
fi

echo OK
